//	File:		ContextRezTypes.r
//	Copyright:	 1993 Apple Computer, Inc. All rights reserved.
//	Author:		Victor J. Hnyp
//	Date:		08-Apr-93

// Revisions
//
//	04/98/93	VJH	2.13	Moved isFindDialog, isFindMoreDialog back to Dialog module
//
//	03/13/93	VJH	2.12	Moved isFindDialog, isFindMoreDialog to Window module
//
//	03/11/93	VJH	2.11	Restored isGuestAccess check to original functionality
//
//	03/11/93	VJH	2.10	Added: seeFolders, seeFiles, makeChanges to isGuestAccess check.
//
//	03/10/93	VJH	2.09	Reimplemented isVideoHWBitDepth the way it was originally defined.
//
//	02/24/93	VJH	2.08	Added new isShareWindowOpen and isShareWindowActive context checks.
//							Removed isSharedOwned context check.
//
//	02/08/93	VJH	2.07	Fixes to isComputerNamed, isFileExists.
//
//	01/18/93	VJH	2.06	Fixed AppleTalk problems with Chooser.  Added isFileExists.

//	01/06/93	VJH	2.05	Converted all context checks to take advantage of Switch and Case statements
//							of Rez - Allows format to omit sections of template

//	01/04/93	VJH	2.04	Changed CopyRight Notice, fixes to System context

//	12/29/92	VJH	2.03	Added definitions for Chooser and System Contexts
//							Fixed Gestalt Check definition context
//							Added definition-dependent #includes and #defines

//	11/29/92	VJH	2.02	Added definitions for File Context

//	11/20/92	VJH	2.01	Added definitions for Dialog Context for inVisible, FindDialog, FindMoreDialog,
//							and isAnyDialogActive.

//	11/16/92	GB	2.00	New, Initial Check in


#ifndef __RENODATA__
	#include "RenoData.h"			// Common with .r files
#endif

#ifndef kCntxChooser
	#define kCntxChooser		'CHSR'
#endif

#ifndef kCntxDialog
	#define kCntxDialog			'DLOG'
#endif

#ifndef kCntxFile
	#define kCntxFile			'FILE'
#endif

#ifndef kCntxFinder
	#define kCntxFinder			'FNDR'
#endif

#ifndef kCntxProcess
	#define kCntxProcess		'PCSS'
#endif

#ifndef kCntxSystem
	#define kCntxSystem			'SYST'
#endif

#ifndef kCntxWindow
	#define kCntxWindow			'WIND'
#endif


		// All structures consists of 5 longs and 2 StringSpecs.
		// Total length is 20 bytes + strings.
		// Format:
		//		longint;		// REQUIRED (but not passed to module)
		//						// 4-character module ID - One of:
		//							'CHSR' - Chooser Module
		//							'DLOG' - Dialog Module
		//							'FILE' - File Module
		//							'FNDR' - Finder Module
		//							'PCSS' - Process Module
		//							'SYST' - System Module
		//							'WIND' - Window Module
		//		longint;		// REQUIRED (by each module)
		//							Function Selector within the module
		//		longint;		// anything
		//		longint;		// StringCompareMethod of StringSpec
		//		pstring;		// String of StringSpec (variable len.)
		//		align word;
		//		longint;		// StringCompareMethod of StringSpec
		//		pstring;		// String of StringSpec (variable len.)
		// The pstring can be any variable-length structure.
		// We go to this trouble for two reasons:
		//	1. To be able to write a ResEdit TMPL for localization.
		//	2. To encapsulate a "universal" context test spec.
		//
		// Reno knows nothing about the content of the context
		// check resource except that the first long is the event
		// id.  The rest of the resource is passed to the context
		// check module.  To add a new context check
		//		1. Write the context check module.
		//		2. Add a new case to kResContext type definition.
		//		3. Write the kResContext resource for the check.
		//		4. Re-Rez the data with the new context checks.

type kResContext {
	longint none=kTargetNone,				// target Process signature
			self=kTargetSelf,
			front=kTargetFront,
			topic=kTargetTopic,
			reno=kTargetReno;
	switch {
		case Chooser:
			key longint = kCntxChooser;
			switch {
				case isPrinterDirect:
					key longint = 0;
					fill byte[16];			// filler for 3 longs & 2 StringSpecs
					
				case isPrintersOnZone:
					key longint = 1;
					longint	equals=2,
							notEqualTo=3,
							greaterThan=4,
							lessThan=5,
							greaterThanOrEqualTo=6,
							lessThanOrEqualTo=7;	// Number compare selector
					longint;				// Compare value
					fill byte[8];			// filler for 1 long & 2 StringSpecs
					
				case isPrinterSerial:
					key longint = 2;
					longint	ModemPort=0,
							PrinterPort=1,
							EitherPort=2;
					fill byte[12];			// filler for 2 longs & 2 StringSpecs
					
				case isPrinterSCSI:
					key longint = 3;
					fill byte[16];			// filler for 3 longs & 2 StringSpecs
					
				case isPrinterType:
					key longint = 4;
					fill byte[4];			// not used
					longint	containsAnything=8,
							empty=9,
							matches=10,
							contains=11,
							startsWith=12,
							endsWith=13;	// StringCompareMethod
					pstring;				// Printer name (variable-length)
					align word;
					fill byte[6];			// filler for 1 long & 1 StringSpec
					
				case isAppleTalk:
					key longint = 5;
					longint	off=0,
							on=1;
					fill byte[12];			// filler for 2 longs & 2 StringSpecs
					
				case isNumberOfZones:
					key longint = 6;
					longint	equals=2,
							notEqualTo=3,
							greaterThan=4,
							lessThan=5,
							greaterThanOrEqualTo=6,
							lessThanOrEqualTo=7;	// Number compare selector
					longint;				// Compare value
					fill byte[8];			// filler for 1 long & 2 StringSpecs
					
				case isPortSelected:
					key longint = 7;
					longint	ModemPort=0,
							PrinterPort=1,
							EitherPort=2;
					fill byte[12];			// filler for 2 longs & 2 StringSpecs
					
				case isPortInUse:
					key longint = 8;
					longint	ModemPort=0,
							PrinterPort=1,
							EitherPort=2;
					fill byte[12];			// filler for 2 longs & 2 StringSpecs
			};	// Chooser switch
			
		case Dialog:
			key longint = kCntxDialog;
			switch {
				case isFront:
					key longint = 0;
					longint;				// Resource ID
					fill byte[12];			// filler for 2 longs & 2 StringSpecs
					
				case isOpen:
					key longint = 1;
					longint;				// Resource ID
					fill byte[12];			// filler for 2 longs & 2 StringSpecs
					
				case isInvisible:
					key longint = 2;
					longint;				// Resource ID
					fill byte[12];			// filler for 2 longs & 2 StringSpecs
					
				case isFindDialog:
					key longint = 3;
					fill byte[16];			// filler for 3 longs & 2 StringSpecs
					
				case isFindMoreDialog:
					key longint = 4;
					fill byte[16];			// filler for 3 longs & 2 StringSpecs
					
				case isAnyDialogActive:
					key longint = 5;
					fill byte[16];			// filler for 3 longs & 2 StringSpecs
			};	// Dialog switch

		case File:
			key longint = kCntxFile;
			switch {
				case isFrontStartup:
					key longint = 0;
					fill byte[16];			// filler for 3 longs & 2 StringSpecs
					
				case isOpenStartup:
					key longint = 1;
					fill byte[16];			// filler for 3 longs & 2 StringSpecs
					
				case isCDStartup:
					key longint = 2;
					fill byte[16];			// filler for 3 longs & 2 StringSpecs
					
				case isPrintMonitor:
					key longint = 3;
					fill byte[16];			// filler for 3 longs & 2 StringSpecs
					
				case isAppleShare:
					key longint = 4;
					fill byte[16];			// filler for 3 longs & 2 StringSpecs
					
				case isEtherTalk:
					key longint = 5;
					fill byte[16];			// filler for 3 longs & 2 StringSpecs
					
				case isTokenTalk:
					key longint = 6;
					fill byte[16];			// filler for 3 longs & 2 StringSpecs
					
				case isControlPanel:
					key longint = 7;
					fill byte[8];			// filler for 2 longs
					pstring;				// Name of Control Panel
					align word;
					fill byte[6];			// filler for 1 long & 1 StringSpec
					
				case isFileExists:
					key longint = 9;
					longint	SystemFolder='macs',
							DesktopFolder='desk',
							TrashFolder='trsh',
							EmptyTrashFolder='empt',
							PrintMonitorFolder='prnt',
							StartupFolder='strt',
							AppleMenuFolder='amnu',
							ControlPanelFolder='ctrl',
							ExtensionsFolder='extn',
							PreferencesFolder='pref',
							TemporaryFolder='temp';		// System folder selector
					fill byte[4];			// filler for 1 long
					pstring;				// Name of desired file
					align word;
					fill byte[6];			// filler for 1 long & 1 StringSpec
			};	// File switch

		case Finder:
			key longint = kCntxFinder;
			fill long;						// Finder selector
			fill byte[16];					// filler for 3 longs & 2 StringSpecs
			
		case Process:
			key longint = kCntxProcess;
			switch {
				case isFront:
					key longint = 0;
					longint;				// Process signature
					fill byte[12];			// filler for 2 longs & 2 StringSpecs
					
				case isOpen:
					key longint = 1;
					longint;				// Process signature
					fill byte[12];			// filler for 2 longs & 2 StringSpecs
			};	// Process switch

		case System:
			key longint = kCntxSystem;
			switch {
				case gestaltCheck:
					key longint = 0;
					longint;				// Gestalt Selector
					longint	equals=2,
							notEqualTo=3,
							greaterThan=4,
							lessThan=5,
							greaterThanOrEqualTo=6,
							lessThanOrEqualTo=7;	// Number compare selector
					fill byte[2];			// not used
					longint;				// Gestalt compare value
					fill byte[2];			// filler for 1 StringSpec
					
				case menuItemExists:
					key longint = 1;
					fill byte[4];			// not used
					longint	containsAnything=8,
							empty=9,
							matches=10,
							contains=11,
							startsWith=12,
							endsWith=13;	// StringCompareMethod
					pstring;				// Menu name (variable-length)
					align word;
					longint	containsAnything=8,
							empty=9,
							matches=10,
							contains=11,
							startsWith=12,
							endsWith=13;	// StringCompareMethod
					pstring;				// Menu Item name (variable-length)
					align word;
					
				case menuItemMarked:
					key longint = 2;
					fill byte[4];			// not used
					longint	containsAnything=8,
							empty=9,
							matches=10,
							contains=11,
							startsWith=12,
							endsWith=13;	// StringCompareMethod
					pstring;				// Menu name (variable-length)
					align word;
					longint	containsAnything=8,
							empty=9,
							matches=10,
							contains=11,
							startsWith=12,
							endsWith=13;	// StringCompareMethod
					pstring;				// Menu Item name (variable-length)
					align word;
					
				case menuItemEnabled:
					key longint = 3;
					fill byte[4];			// not used
					longint	containsAnything=8,
							empty=9,
							matches=10,
							contains=11,
							startsWith=12,
							endsWith=13;	// StringCompareMethod
					pstring;				// Menu name (variable-length)
					align word;
					longint	containsAnything=8,
							empty=9,
							matches=10,
							contains=11,
							startsWith=12,
							endsWith=13;	// StringCompareMethod
					pstring;				// Menu Item name (variable-length)
					align word;
					
				case menuItemDisabled:
					key longint = 4;
					fill byte[4];			// not used
					longint	containsAnything=8,
							empty=9,
							matches=10,
							contains=11,
							startsWith=12,
							endsWith=13;	// StringCompareMethod
					pstring;				// Menu name (variable-length)
					align word;
					longint	containsAnything=8,
							empty=9,
							matches=10,
							contains=11,
							startsWith=12,
							endsWith=13;	// StringCompareMethod
					pstring;				// Menu Item name (variable-length)
					align word;
					
				case isFileSharing:
					key longint = 6;
					longint	off=0,
							on=1;
					fill byte[12];			// filler for 2 longs & 2 StringSpecs
					
				case isVideoHWBitDepth:
					key longint = 7;
					longint	equals=2,
							notEqualTo=3,
							greaterThan=4,
							lessThan=5,
							greaterThanOrEqualTo=6,
							lessThanOrEqualTo=7;	// Number compare selector
					longint;				// Compare value
					fill byte[8];			// filler for 1 long & 2 StringSpecs
					
				case isMonitorBitDepth:
					key longint = 8;
					longint	equals=2,
							notEqualTo=3,
							greaterThan=4,
							lessThan=5,
							greaterThanOrEqualTo=6,
							lessThanOrEqualTo=7;	// Number compare selector
					longint;				// Compare value
					fill byte[8];			// filler for 1 long & 2 StringSpecs
					
				case isComputerNamed:
					key longint = 9;
					fill byte[4];			// not used
					longint	containsAnything=8,
							empty=9,
							matches=10,
							contains=11,
							startsWith=12,
							endsWith=13;	// StringCompareMethod
					pstring;				// Printer name (variable-length)
					align word;
					fill byte[6];			// filler for 1 long & 1 StringSpec
					
				case isGuestAccess:
					key longint = 10;
					longint	off=0,
							on=1;
					fill byte[12];			// filler for 2 longs & 2 StringSpecs
					
				case isNumberMonitors:
					key longint = 11;
					longint	equals=2,
							notEqualTo=3,
							greaterThan=4,
							lessThan=5,
							greaterThanOrEqualTo=6,
							lessThanOrEqualTo=7;	// Number compare selector
					longint;				// Compare value
					fill byte[8];			// filler for 1 long & 2 StringSpecs
			};	// System switch

		case Window:
			key longint = kCntxWindow;
			switch {
				case isFront:
					key longint = 0;
					fill byte[4];			// not used
					longint	containsAnything=8,
							empty=9,
							matches=10,
							contains=11,
							startsWith=12,
							endsWith=13;	// StringCompareMethod
					pstring;				// Window name (variable-length)
					align word;
					fill byte[6];			// filler for 1 long & 1 StringSpec
					
				case isOpen:
					key longint = 1;
					fill byte[4];			// not used
					longint	containsAnything=8,
							empty=9,
							matches=10,
							contains=11,
							startsWith=12,
							endsWith=13;	// StringCompareMethod
					pstring;				// Window name (variable-length)
					align word;
					fill byte[6];			// filler for 1 long & 1 StringSpec
					
				case isInvisible:
					key longint = 2;
					fill byte[4];			// not used
					longint	containsAnything=8,
							empty=9,
							matches=10,
							contains=11,
							startsWith=12,
							endsWith=13;	// StringCompareMethod
					pstring;				// Window name (variable-length)
					align word;
					fill byte[6];			// filler for 1 long & 1 StringSpec
					
				case isShareWindowOpen:
					key longint = 3;
					fill byte[16];			// filler for 3 longs & 2 StringSpecs
					
				case isShareWIndowActive:
					key longint = 4;
					fill byte[16];			// filler for 3 longs & 2 StringSpecs

				case isCollapsed:
					key longint = 5;
					fill byte[16];			// filler for 3 longs & 2 StringSpecs
			};	// Window switch

	}; // switch
}; // type
