// Copyright 1994 Apple Computer, Inc. All rights reserved.
// Author:	John R. Powers, III
// Date:    04-Feb-93

// ============================= RenoData.h ===========================
//
// Constants for Reno data resources.
// Also used by RenoTheApp

#ifndef __RENODATA__
#define __RENODATA__

/*
	27-May-92	extract application constants, put into RenoDefinitions.
				return control constants to this file.
				return Howdy string constants to this file.
	28-Jun-92	add kHuhBtnEventId, kAppLogoId, kAppTextId.
	11-Aug-92	Added kResKeyDump and kResKeyDumpLabel for the key dump structure.
	13-Aug-92	Moved the kSliderScope here from RenoDefinitions.h.
	17-Aug-92	Add kTargetReno ('r***')
	02-Sep-92	Add kResMaskGroup, kResSTxtGroup, kResSTxtTransGroup.
				Add kResSTxtTrans objects and text suffixes.
	06-Oct-92	Add kResSTxt0-9 and kResSTxtTrans0-9
				Add new RenoDB resource types.
				Change kAppRelease from 32 (1.0a3) to 35 (1.0a6)
				Change kDataVersionNumber from 1.0d2 to 1.0a6.
				Add kDataHeaderType ('<DF>') and its label.
	16-Nov-92	GB Removed "kCntx" defines, moved to Context.h in the Reno:Modules folder
	17-Dec-92	Add kNotDefinedV to AccWin views.
	24-Jan-93	Add kResMovieB ('pcMB')
	29-Jan-93	Rename kAppRelease as kAppMinSeqNum.
				Add kResVersionInfo resource.
				Move kAppMinSeqNum from the kDataVersionBytes
				used in the database 'vers' resources
				to the kResVersionInfo resource.  
				Revise howdy string id numbers (from STR# to TEXT.)
				Add kVersionInfoAppID and kVersionInfoDataID.
	02-Feb-93	Changed Howdy TEXT ID's from 1002 and 1003 to 502 and 503.
	20-Feb-93	Add kResPanelTopic ('ppPT').
	23-Feb-93	Add kMinV and kMaxV constants to represent the minimum and
				maximum view numbers used to start Reno.
	24-Feb-93	Added GoStartButton defines.
	03-Mar-93	Remove kDummyV (-2) and kQuesV (-1).
	24-May-93	Added kHandlerMapResType and kHandlerMapResTypeLabel.

// 22-Sep-93 1.1d1e7 <2.14> PVC
// Move	kResPanelTopicSub define to RenoData.h.
//
// 05-Oct-93 1.1d1e10 <2.15> JRP
// Add kResSTxtAlign panel object and kResSTxtAlignLabel.
// Add kResSTxtAlignTrans panel object.
// Add kResActionOnShow and kResActionOnHide panel objects.
// Add kResPanelTopicBra panel.
//
// 19-Oct-93 1.1d1e11 <2.16> JRP
// Add kResActionOnCreate.
// Add kResActionOnDelete.
// Add kResActionOnShowOnce.
// Add kResActionOnHideOnce.
//
// 26-Oct-93 1.1d2e2 <2.14> JRP
// Add kTargetEventList.
//
// 08-Nov-93 1.1d2e6 <2.15> JRP
// Add kResCtrlItemIdIsRoot.
//
// 04-Feb-94 1.2b3e2 <3.01> JRP
// Add #if __WantScrollingText__ to include/exclude TPoSTxtScroll
// andTPoSTxtScrollFormat. This neat feature is not accessible by authors.
//
// 04-Feb-94 1.2b6e3 <3.02> PVC
// Add drop in object resources.
//
// 04-May-94 1.2b12e3 <3.03> PVC
// Add default content check topic resource type (kResTopicContextChecks).
//
// 08-May-94 1.2b13e1 JRP
// Change kCopyright and kProgramName
// to kDataCopyright and kDataProgramName
// to avoid label conflicts in the 'vers' resource.
//
// 23-May-94 1.2b16e1 PVC
// Add kSliderDirectionScope, an optional resource that specifies whether the slider
// is right to left or left to right ( default no resource is left to right ).


*/

										// Feature selection

								// Set __WantScrollingText__ to 1
								// to include the scrolling text panel object.
								// This flag must be set in
								// RenoData.h and RenoDefinitions.h.

#define __WantScrollingText__	0

 				// Version information for App (Reno)
				// This is the oldest (earliest) compatible app version.
						// kAppMinSeqNum = 29 + app alpha release.
						// EG: 29 + a1 = 30, 29 + a14 = 43.
						// This information goes into the kResVersionInfo
						// resource in the database.
						// 05-Feb-93 1.0a14 JRP	New.

#define kAppMinVersionString	"1.0 or newer"
#define kAppMinSeqNum			52

				// Default version information for DATA

#define kDataVersionString  "1.0"
#define kDataRelease		1
#define kDataRevision		0
#define kDataInternalStage	0
#define kDataVersionDate	"21-Jul-93"
#define	kDataVersionBytes	kDataRelease, kDataRevision, alpha, kDataInternalStage, verUs
#define kDataProgramName	"Apple Guide"
#define	kDataCopyright		"1994 Apple Computer,Inc. All rights reserved."

										// Resource types (2nd letter)
#define kResTypePanel		'p'
#define kResTypeQualifier	'q'
#define kResTypeContent		'c'
#define kResTypeContext		'x'
#define	kResTypeAction		'a'
#define kResTypeFormat		'f'

										// Mask for first three letters of type
#define kResMaskGroup		0xFFFFFF00
										// Database version information
#define kResVersionInfo		'acVI'

										// Access data resources

#define kIndexScope			'acIN'
#define kStartupScope		'acSU'
#define kAreasScope			'acTA'
#define kResTopicL			'acTL'
										// New (1.0a6) RenoDB AccWin resource types.
										// The authoring tool does not produce these
										// resources, but the conversion tool does.

#define	kTopicResType			'acTB'
#define	kTopicResTypeLabel		"acTB"

#define kQuestionResType		'acQU'
#define kQuestionResTypeLabel	"acQU"
										// Data handler mapping resource
#define kHandlerMapResType 		'dhMP'

										// Pseudo-type for a 'TMPL' to edit
										// the data fork using Resourcerer 1.1.1.
										// Also set Resourcerer file preferences
										// to edit the data fork with this label.
#define kDataHeaderTypeLabel	"<DF>"

										// Slider data
#define	kSliderScope			'acSD'
#define	kSliderDirectionScope	'acSF'

										// database scope for topic
#define kResTopic			'acPT'
										// database scope for topic default context checks resource
#define kResTopicContextChecks	'acCC'		
										// database scopes for panel data
#define kResPanelUnivA		'ppUA'
#define kResPanel			'ppUA'
#define kResPanelTopic		'ppPT'
#define kResPanelTopicSub	'ppTS'		// launch sub-sequence
#define kResPanelTopicBra	'ppTB'		// launch branch-sequence

										// database key dump type
#define	kResKeyDump			'keys'

										// Resources for a topic

										// Panel qualifiers in the topic script:

#define kResMakeSure		'pqMS'			// Make-sure
#define kShowIf				'pqSH'			// Qualify with a show condition.
#define kSkipIf				'pqSK'			// Qualify with a skip condition.

										// Other commands in the topic script:

#define kSetNewTopicTarget	'pqNT'			// Set new target for the topic.

										// Conditions:

#define kResCondition		'pqCN'			// Where we start and for chaining.
#define kResContext			'pxCA'			// Context
											// Control item (see below)

										// Things that go in the panel:

#define kResCtrlItem		'pcCI'			// Radio button or checkbox

#define	kResGraphic			'pcGR'			// pcGR is an alias for PICT
#define	kResGraphicF		'pcGF'			// pcGF is a formatted pcGR

#define kResHot				'pcHA'			// Hot area

#define kResMovie			'pcMO'			// Movie with no badge or controller.
#define kResMovieB			'pcMB'			// Movie with badge visible.
#define kResMovieC			'pcMC'			// Movie with controller visible.

#define kResPrompt			'pcPR'			// Prompt

#define kResRenoBtn			'pcRB'			// Reno Button

#if __NEW13DROPIN__
#define	kResDropIn			'pdOB'			// Reno drop in object data resource type
#define	kResDropInMask		(0xFFFF0000)
#define	kResDropInGroup		(kResDropIn & kResDropInMask)
#endif

#define kResActionOnCreate		'pcAC'		// Action on creation
#define kResActionOnDelete		'pcAD'		// Action on deletion
#define kResActionOnShow		'pcAS'		// Action on "Show"
#define kResActionOnHide		'pcAH'		// Action on "Hide"
#define kResActionOnShowOnce	'pcAW'		// Action on first "Show"
#define kResActionOnHideOnce	'pcAE'		// Action on first "Hide"

							// Optionally include/exclude TPoSTxtScroll
							// and TPoSTxtScrollFormat from the build.
#if __WantScrollingText__
#define kResSTxtS			'pcSX'			// General scrolling text
#define kResSTxtSF			'pcSF'			// Formatted scrolling text
#endif
											// Text group
#define kResSTxt			'pcTX'			// General styled text
#define kResSTxtAlign		'pcTA'			// General text with alignment
#define kResSTxtB			'pcTB'			// Text in body format
#define kResSTxtT			'pcTT'			// Text in tag format
#define kResSTxtF			'pcTF'			// Text in full panel format
#define kResSTxtC			'pcTC'			// Text in custom format
#define kResSTxtGroup		(kResSTxt & kResMaskGroup)

											// Transparent text group
#define kResSTxtTrans		'pcVX'			// General styled text on trans.
#define kResSTxtAlignTrans	'pcVA'			// Transparent text with alignment
#define kResSTxtBTrans		'pcVB'			// Text in body format
#define kResSTxtTTrans		'pcVT'			// Text in tag format
#define kResSTxtFTrans		'pcVF'			// Text in full panel format
#define kResSTxtCTrans		'pcVC'			// Text in custom format
#define kResSTxtTransGroup	(kResSTxtTrans & kResMaskGroup)
											
											// Text-type suffixes
#define kResSTxtSuffixGen	   'X'
#define kResSTxtSuffixAlign	   'A'
#define kResSTxtSuffixBody	   'B'
#define kResSTxtSuffixTag	   'T'
#define kResSTxtSuffixFull	   'F'
#define kResSTxtSuffixCust	   'C'
											// Formatted text
#define kResSTxt0			'pcT0'			// Text using format 'pfFA' ID=kPreFormatVariable+0
#define kResSTxt1			'pcT1'			// Text using format 'pfFA' ID=kPreFormatVariable+1
#define kResSTxt2			'pcT2'			// Text using format 'pfFA' ID=kPreFormatVariable+2
#define kResSTxt3			'pcT3'			// Text using format 'pfFA' ID=kPreFormatVariable+3
#define kResSTxt4			'pcT4'			// Text using format 'pfFA' ID=kPreFormatVariable+4
#define kResSTxt5			'pcT5'			// Text using format 'pfFA' ID=kPreFormatVariable+5
#define kResSTxt6			'pcT6'			// Text using format 'pfFA' ID=kPreFormatVariable+6
#define kResSTxt7			'pcT7'			// Text using format 'pfFA' ID=kPreFormatVariable+7
#define kResSTxt8			'pcT8'			// Text using format 'pfFA' ID=kPreFormatVariable+8
#define kResSTxt9			'pcT9'			// Text using format 'pfFA' ID=kPreFormatVariable+9
											
											// Formatted transparent text
#define kResSTxtTrans0		'pcV0'			// Text using format 'pfFA' ID=kPreFormatVariable+0
#define kResSTxtTrans1		'pcV1'			// Text using format 'pfFA' ID=kPreFormatVariable+1
#define kResSTxtTrans2		'pcV2'			// Text using format 'pfFA' ID=kPreFormatVariable+2
#define kResSTxtTrans3		'pcV3'			// Text using format 'pfFA' ID=kPreFormatVariable+3
#define kResSTxtTrans4		'pcV4'			// Text using format 'pfFA' ID=kPreFormatVariable+4
#define kResSTxtTrans5		'pcV5'			// Text using format 'pfFA' ID=kPreFormatVariable+5
#define kResSTxtTrans6		'pcV6'			// Text using format 'pfFA' ID=kPreFormatVariable+6
#define kResSTxtTrans7		'pcV7'			// Text using format 'pfFA' ID=kPreFormatVariable+7
#define kResSTxtTrans8		'pcV8'			// Text using format 'pfFA' ID=kPreFormatVariable+8
#define kResSTxtTrans9		'pcV9'			// Text using format 'pfFA' ID=kPreFormatVariable+9
											
										// In PreWin, but not in panel:

#define kResPreWinBtn		'pwRB'			// PreWin Button

										// Coach's Chalkboard mark
#define kResCoachMark		'paCM'

										// Format
#define kResFormat			'pfFA'

										// Actions

#define kResEvent			'paEV'
#define kHuhTopic			'paHU'

										// Labels for ResEdit TMPL's.
										// These are the "real" resources.
										// Keep in alphabetical order.

#define kIndexScopeLabel		"acIN"
#define kResTopicLabel			"acPT"
#define	kSliderScopeLabel		"acSD"
#define kStartupScopeLabel		"acSU"
#define kAreasScopeLabel		"acTA"
#define kResTopicLLabel			"acTL"
#define kResVersionInfoLabel	"acVI"
#define kHandlerMapResTypeLabel	"dhMP"
#define kResCoachMarkLabel		"paCM"
#define kResEventLabel			"paEV"
#define kHuhTopicLabel			"paHU"
#define kResCtrlItemLabel		"pcCI"
#define kResMovieCLabel			"pcMC"
#define kResMovieLabel			"pcMO"
#define	kResGraphicLabel		"pcGR"
#define kResHotLabel			"pcHA"
#define kResPromptLabel			"pcPR"
#define kResRenoBtnLabel		"pcRB"
#define kResSTxtLabel			"pcTX"
#define kResSTxtAlignLabel		"pcTA"
#define kResFormatLabel			"pfFA"
#define kResPanelUnivALabel		"ppUA"
#define kResPanelLabel			"ppUA"
#define kResPanelTopicLabel		"ppPT"
#define kResConditionLabel		"pqCN"
#define kResMakeSureLabel		"pqMS"
#define kResPreWinBtnLabel		"pwRB"
#define kResContextLabel		"pxCA"
#define kResKeyDumpLabel		"keys"
										// Selectors for context data.
										// Used only for data keys and
										// passed to context checker.
										// kResCondition logic
#define	kNone		0
#define kOr			1
#define kAnd		2
#define kNot		1
#define kDontUpdate		0
#define kUpdate			1
										// The "use-default" flag.
#define kDefault		-1
										// No maximum window height flag.
#define kNoMaxHeight	9999
										// Target options
#define kTargetNone			'n***'
#define kTargetSelf			's***'
#define kTargetFront		'f***'
#define kTargetTopic		't***'
#define kTargetReno			'r***'
#define kTargetEventList	'L***'

										// Reno's version information resource
#define kVersionInfoDataID		146
#define kVersionInfoAppID		147
										// Startup preferences
#define kStartPrefsID		146
										// Resource Id's (DBRefs) for access data
#define kIndexScopeID		144
#define kAreasScopeID		145

										// AccWin topic header strings
#define kResHeaderStr		'STR#'
#define kHeaderStrResID		1002
										// Access Window views
										// (for internal and author use)
										// WARNING:	If you add a view constant,
										// make sure that you change kMaxV!!!
#define kNotDefinedV	-1
#define	kHowdyV			1
#define	kAreasV			2
#define	kIndexV			3
#define	kLookFV			4
#define kSingleHowdyV	5
#define kSingleTopicV	6

#define kMinV			kHowdyV			// Lowest possible view number
#define kMaxV			kSingleTopicV	// Highest possible view number

										// PreWin format resource id's
#define kPreFormatFull		150
#define kPreFormatTag		151
#define kPreFormatBody		152
#define kPreFormatCustom	153
#define kPreFormatScroll	154
										// Starting id for variable formats
#define kPreFormatVariable	160
										// Huh? button id
#define kHuhBtnId			200
#define kHuhBtnEventId		200
										// Go Start button id
#define kGoStartBtnId		201
#define kGoStartBtnEventId	201
										// App logo and title id
#define kAppLogoId			501
#define kAppTextId			501
										// AccWin Howdy view strings
										// These are TEXT resource ID's.
#define kHowdyFullStrID		502
#define kHowdyHalfStrID		503

										// These are internally-set
										// "control" values accessible by
										// the author's script.
#define kResCtrlItemIdIsRoot	301

		// The following controls constants are copied from Controls.h
		// They are used by RenoDataTypes.r and RenoData.r as resource
		// keys then passed on to NewControl as the procID.
		
		// The default is to use the window font (proc + useWFont).
		// See IM-I page I-315.

#define pushButProcRenoSysFont		0
#define checkBoxProcRenoSysFont		1
#define radioButProcRenoSysFont		2

#define pushButProcReno				8
#define checkBoxProcReno			9
#define radioButProcReno			10

#endif __RENODATA__
